#!/usr/bin/python

"""
main.py - Main entry point for game
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *

import resources
import game
import cutscenes

# Start
def main(mainpath):

    # Initialise pygame
    pygame.init()
    pygame.mixer.init()
    pygame.mouse.set_visible(False)
    
    # start up director
    framerate = 30
    #screen_res = (480,320)
    screen_res = (320,240)
    #screen_res = (192,128)
    window_title = "Game"
    dir = GameDirector(window_title, screen_res, framerate)
    
    # Load resources
    resources.init(mainpath,screen_res)
    
    # Title Screen
    titlescene = cutscenes.TitleScreen(dir, screen_res)
    titlescene.savepath = os.path.join(mainpath,'data','saved_progress.txt')
    dir.addscene('titlescene', titlescene)
    
    # Cutscens
    logocutscene = cutscenes.CutScene(dir, screen_res)
    dir.addscene('logocutscene', logocutscene)
    
    storycutscene = cutscenes.CutScene(dir, screen_res)
    dir.addscene('storycutscene', storycutscene)
    
    gotscrollscene = cutscenes.GotScroll(dir, screen_res)
    dir.addscene('gotscroll', gotscrollscene)
    
    wingamescene = cutscenes.CutScene(dir, screen_res)
    dir.addscene('wingamescene', wingamescene)
    
    # Load game scenes
    maingame = game.MainGame(dir, screen_res)
    maingame.progress_data.savepath = os.path.join(mainpath,'data','saved_progress.txt')
    titlescene.h_progress_data = maingame.progress_data
    dir.addscene('maingame', maingame)
    deathscene = game.Death(dir, screen_res)
    dir.addscene('death', deathscene)
    pausescene = game.PauseScreen(dir, screen_res)
    dir.addscene('pausescene', pausescene)
    maingame.h_pausescene = pausescene
    
    # start up director
    #dir.change_scene('maingame', [True,"level_computerroom",-1])
    #dir.change_scene('maingame', [True,"level_dragonroom",-1])
    
    #dir.change_scene('maingame', [True,"level_basicmechanisms",-1])
    #dir.change_scene('maingame', [True,"level_basiclava",-1])
    #dir.change_scene('maingame', [True,"level_theroundabout",-1])
    #dir.change_scene('maingame', [True,"level_basicelectric",-1])
    #dir.change_scene('maingame', [True,"level_thelift",-1])
    
    #dir.change_scene('maingame', [True,"level_getscroll_blue",-1])
    #dir.change_scene('maingame', [True,"level_getscroll_green",-1])
    #dir.change_scene('maingame', [True,"level_getscroll_red",-1])
    
    #dir.change_scene('titlescene', [])
    dir.change_scene('logocutscene', resources.logocutscene_data)
    #dir.change_scene('storycutscene', resources.storycutscene_data)
    #dir.change_scene('wingamescene', resources.wingamescene_data)
    
    #dir.change_scene('gotscroll', ["scroll_red"])
    dir.loop()
    
    # exiting, record framerate
    #fp = open(os.path.join(mainpath,'framerate.txt'),"w")
    #fp.write("%f\n"%(maingame.avgframerate))
    #fp.close()

