#!/usr/bin/python

"""
player.py - player objects
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *
from math import *

import resources

class KeyControl(object):
    def __init__(self,controlmap):
        self.CM = controlmap
        self.u = 0
        self.d = 0
        self.l = 0
        self.r = 0
        self.jump = False
        self.use = False
        self.pausemenu = False
    
    def ProcessKeyEvent(self,event):
        if event.type == KEYDOWN:
            if event.key == self.CM["U"]:
                self.u = 1
            elif event.key == self.CM["D"]:
                self.d = 1
            elif event.key == self.CM["L"]:
                self.l = 1
            elif event.key == self.CM["R"]:
                self.r = 1
            if event.key == self.CM["Jump"]:
                self.jump = True
            if event.key == self.CM["Use"]:
                self.use = True
        elif event.type == KEYUP:
            if event.key == self.CM["U"]:
                self.u = 0
            elif event.key == self.CM["D"]:
                self.d = 0
            elif event.key == self.CM["L"]:
                self.l = 0
            elif event.key == self.CM["R"]:
                self.r = 0
            elif event.key == self.CM["Use"]:
                self.use = False

class Player(object):
    def __init__(self):
        
        self.x = -1
        self.y = -1
        self.ground_state = True
        self.vy = 0
        self.heading = 'right'
        
        self.gaittype = 'stand'
        self.gait = 0
        self.gait_to = 5
        
        self.sizex = 20
        self.sizey = 28
        self.speed = 4
        
        self.sx = 16
        
        self.control = KeyControl(resources.controlmap)
        self.exiting = False
        self.dying = False
        self.destination = []
    
    def set_handles(self,handles):
        (self.h_camera, self.h_tiledlayers) = handles
    
    def kill(self):
        #pass
        self.dying = True
    
    def HandleObjectWallCollision(self,objpos,objmovevec):
        
        x = objpos[0]
        y = objpos[1]
        vx = objmovevec[0]
        vy = objmovevec[1]
        
        tileocc_ul = self.h_tiledlayers.map_size[0]*((y-11)/16)+((x-(self.sx/2))/16)
        tileocc_ur = self.h_tiledlayers.map_size[0]*((y-11)/16)+((x+(self.sx/2))/16)
        tileocc_ll = self.h_tiledlayers.map_size[0]*((y+15)/16)+((x-(self.sx/2))/16)
        tileocc_lr = self.h_tiledlayers.map_size[0]*((y+15)/16)+((x+(self.sx/2))/16)
        
        tileocc_ml = self.h_tiledlayers.map_size[0]*(y/16)+((x-(self.sx/2))/16)
        tileocc_mr = self.h_tiledlayers.map_size[0]*(y/16)+((x+(self.sx/2))/16)
        
        if vx > 0 and self.h_tiledlayers.occlayer[tileocc_mr]:
            x = 16*(tileocc_ur % self.h_tiledlayers.map_size[0]) - 1 - (self.sx/2)
        if vx < 0 and self.h_tiledlayers.occlayer[tileocc_ml]:
            x = 16*(tileocc_ul % self.h_tiledlayers.map_size[0]) + 16 + 1 + (self.sx/2)
        
        if vx > 0 and vy == 0:
            if self.h_tiledlayers.occlayer[tileocc_ur] or self.h_tiledlayers.occlayer[tileocc_lr]:
                x = 16*(tileocc_ur % self.h_tiledlayers.map_size[0]) - 1 - (self.sx/2)
        elif vx < 0 and vy == 0:
            if self.h_tiledlayers.occlayer[tileocc_ul] or self.h_tiledlayers.occlayer[tileocc_ll]:
                x = 16*(tileocc_ul % self.h_tiledlayers.map_size[0]) + 16 + 1 + (self.sx/2)
        elif vx == 0 and vy > 0:
            if self.h_tiledlayers.occlayer[tileocc_lr] or self.h_tiledlayers.occlayer[tileocc_ll]:
                y = 16*(tileocc_ll/self.h_tiledlayers.map_size[0]) - 1 - 16
        elif vx == 0 and vy < 0:
            if self.h_tiledlayers.occlayer[tileocc_ul] or self.h_tiledlayers.occlayer[tileocc_ur]:
                y = 16*(tileocc_ul/self.h_tiledlayers.map_size[0]) + 16 + 1 + 12
        elif vx > 0 and vy < 0:
            if self.h_tiledlayers.occlayer[tileocc_ul] and self.h_tiledlayers.occlayer[tileocc_lr]:
                x = 16*(tileocc_ur % self.h_tiledlayers.map_size[0]) - 1 - (self.sx/2)
                y = 16*(tileocc_ul/self.h_tiledlayers.map_size[0]) + 16 + 1 + 12
            elif self.h_tiledlayers.occlayer[tileocc_ul]:
                y = 16*(tileocc_ul/self.h_tiledlayers.map_size[0]) + 16 + 1 + 12
            elif self.h_tiledlayers.occlayer[tileocc_lr]:
                x = 16*(tileocc_ur % self.h_tiledlayers.map_size[0]) - 1 - (self.sx/2)
            elif self.h_tiledlayers.occlayer[tileocc_ur]:
                xref = 16*(tileocc_ur % self.h_tiledlayers.map_size[0]) - 1 - (self.sx/2)
                yref = 16*(tileocc_ul/self.h_tiledlayers.map_size[0]) + 16 + 1 + 12
                if fabs(x-xref) < fabs(y-yref) or self.h_tiledlayers.occlayer[tileocc_ur+self.h_tiledlayers.map_size[0]]:
                    x = xref
                else:
                    y = yref
        elif vx > 0 and vy > 0:
            if self.h_tiledlayers.occlayer[tileocc_ll] and self.h_tiledlayers.occlayer[tileocc_ur]:
                x = 16*(tileocc_ur % self.h_tiledlayers.map_size[0]) - 1 - (self.sx/2)
                y = 16*(tileocc_ll/self.h_tiledlayers.map_size[0]) - 1 - 16
            elif self.h_tiledlayers.occlayer[tileocc_ll]:
                y = 16*(tileocc_ll/self.h_tiledlayers.map_size[0]) - 1 - 16
            elif self.h_tiledlayers.occlayer[tileocc_ur]:
                x = 16*(tileocc_ur % self.h_tiledlayers.map_size[0]) - 1 - (self.sx/2)
            elif self.h_tiledlayers.occlayer[tileocc_lr]:
                xref = 16*(tileocc_ur % self.h_tiledlayers.map_size[0]) - 1 - (self.sx/2)
                yref = 16*(tileocc_ll/self.h_tiledlayers.map_size[0]) - 1 - 16
                if fabs(x-xref) < fabs(y-yref) or self.h_tiledlayers.occlayer[tileocc_lr-self.h_tiledlayers.map_size[0]]:
                    x = xref
                else:
                    y = yref
        elif vx < 0 and vy > 0:
            if self.h_tiledlayers.occlayer[tileocc_ul] and self.h_tiledlayers.occlayer[tileocc_lr]:
                x = 16*(tileocc_ul % self.h_tiledlayers.map_size[0]) + 16 + 1 + (self.sx/2)
                y = 16*(tileocc_ll/self.h_tiledlayers.map_size[0]) - 1 - 16
            elif self.h_tiledlayers.occlayer[tileocc_lr]:
                y = 16*(tileocc_ll/self.h_tiledlayers.map_size[0]) - 1 - 16
            elif self.h_tiledlayers.occlayer[tileocc_ul]:
                x = 16*(tileocc_ul % self.h_tiledlayers.map_size[0]) + 16 + 1 + (self.sx/2)
            elif self.h_tiledlayers.occlayer[tileocc_ll]:
                xref = 16*(tileocc_ul % self.h_tiledlayers.map_size[0]) + 16 + 1 + (self.sx/2)
                yref = 16*(tileocc_ll/self.h_tiledlayers.map_size[0]) - 1 - 16
                if fabs(x-xref) < fabs(y-yref) or self.h_tiledlayers.occlayer[tileocc_ll-self.h_tiledlayers.map_size[0]]:
                    x = xref
                else:
                    y = yref
        elif vx < 0 and vy < 0:
            if self.h_tiledlayers.occlayer[tileocc_ur] and self.h_tiledlayers.occlayer[tileocc_ll]:
                x = 16*(tileocc_ul % self.h_tiledlayers.map_size[0]) + 16 + 1 + (self.sx/2)
                y = 16*(tileocc_ul/self.h_tiledlayers.map_size[0]) + 16 + 1 + 12
            elif self.h_tiledlayers.occlayer[tileocc_ur]:
                y = 16*(tileocc_ul/self.h_tiledlayers.map_size[0]) + 16 + 1 + 12
            elif self.h_tiledlayers.occlayer[tileocc_ll]:
                x = 16*(tileocc_ul % self.h_tiledlayers.map_size[0]) + 16 + 1 + (self.sx/2)
            elif self.h_tiledlayers.occlayer[tileocc_ul]:
                xref = 16*(tileocc_ul % self.h_tiledlayers.map_size[0]) + 16 + 1 + (self.sx/2)
                yref = 16*(tileocc_ul/self.h_tiledlayers.map_size[0]) + 16 + 1 + 12
                if fabs(x-xref) < fabs(y-yref) or self.h_tiledlayers.occlayer[tileocc_ul+self.h_tiledlayers.map_size[0]]:
                    x = xref
                else:
                    y = yref
        
        return (x,y)
    
    def HandleBoxCollision(self,objpos,objmovevec,box):
        
        x = objpos[0]
        y = objpos[1]
        vx = objmovevec[0]
        vy = objmovevec[1]
        ts = 16
        
        tileocc_ul = (y-11) < box.y+(ts/2) and (y-11) > box.y-(ts/2) and (x-(self.sx/2)) < box.x+(ts/2) and (x-(self.sx/2)) > box.x-(ts/2)
        tileocc_ur = (y-11) < box.y+(ts/2) and (y-11) > box.y-(ts/2) and (x+(self.sx/2)) < box.x+(ts/2) and (x+(self.sx/2)) > box.x-(ts/2)
        tileocc_ll = (y+15) < box.y+(ts/2) and (y+15) > box.y-(ts/2) and (x-(self.sx/2)) < box.x+(ts/2) and (x-(self.sx/2)) > box.x-(ts/2)
        tileocc_lr = (y+15) < box.y+(ts/2) and (y+15) > box.y-(ts/2) and (x+(self.sx/2)) < box.x+(ts/2) and (x+(self.sx/2)) > box.x-(ts/2)
        
        tileocc_ml = y < box.y+(ts/2) and y > box.y-(ts/2) and (x-(self.sx/2)) < box.x+(ts/2) and (x-(self.sx/2)) > box.x-(ts/2)
        tileocc_mr = y < box.y+(ts/2) and y > box.y-(ts/2) and (x+(self.sx/2)) < box.x+(ts/2) and (x+(self.sx/2)) > box.x-(ts/2)
        
        #print x,y,box.x,box.y
        #print tileocc_ul,tileocc_ur,tileocc_ll,tileocc_lr,tileocc_ml,tileocc_mr
        
        if vx > 0 and tileocc_mr:
            x = box.x-(ts/2) - (self.sx/2) - 1
            #print "b"
        if vx < 0 and tileocc_ml:
            x = box.x+(ts/2) + (self.sx/2) + 1
        
        if vx > 0 and vy == 0:
            if tileocc_ur or tileocc_lr:
                x = box.x-(ts/2) - (self.sx/2) - 1
                #print "a"
        elif vx < 0 and vy == 0:
            if tileocc_ul or tileocc_ll:
                x = box.x+(ts/2) + (self.sx/2) + 1
        elif vx == 0 and vy > 0:
            if tileocc_lr or tileocc_ll:
                y = box.y-(ts/2) - 16 - 1
        elif vx == 0 and vy < 0:
            if tileocc_ul or tileocc_ur:
                y = box.y+(ts/2) + 12 + 1
        elif vx > 0 and vy < 0:
            if tileocc_ul:
                y = box.y+(ts/2) + 12 + 1
            elif tileocc_lr:
                x = box.x-(ts/2) - (self.sx/2) - 1
            elif tileocc_ur:
                xref = box.x-(ts/2) - (self.sx/2) - 1
                yref = box.y+(ts/2) + 12 + 1
                if fabs(x-xref) < fabs(y-yref):
                    x = xref
                else:
                    y = yref
        elif vx > 0 and vy > 0:
            if tileocc_ll:
                y = box.y-(ts/2) - 16 - 1
            elif tileocc_ur:
                x = box.x-(ts/2) - (self.sx/2) - 1
            elif tileocc_lr:
                xref = box.x-(ts/2) - (self.sx/2) - 1
                yref = box.y-(ts/2) - 16 - 1
                if fabs(x-xref) < fabs(y-yref):
                    x = xref
                else:
                    y = yref
        elif vx < 0 and vy > 0:
            if tileocc_lr:
                y = box.y-(ts/2) - 16 - 1
            elif tileocc_ul:
                x = box.x+(ts/2) + (self.sx/2) + 1
            elif tileocc_ll:
                xref = box.x+(ts/2) + (self.sx/2) + 1
                yref = box.y-(ts/2) - 16 - 1
                if fabs(x-xref) < fabs(y-yref):
                    x = xref
                else:
                    y = yref
        elif vx < 0 and vy < 0:
            if tileocc_ur:
                y = box.y+(ts/2) + 12 + 1
            elif tileocc_ll:
                x = box.x+(ts/2) + (self.sx/2) + 1
            elif tileocc_ul:
                xref = box.x+(ts/2) + (self.sx/2) + 1
                yref = box.y+(ts/2) + 12 + 1
                if fabs(x-xref) < fabs(y-yref):
                    x = xref
                else:
                    y = yref
        
        return (x,y)
    
    def ExitCheck(self):
        current_tile = self.h_tiledlayers.map_size[0]*((self.y+4)/self.h_tiledlayers.tilesize)+(self.x/self.h_tiledlayers.tilesize)
        exit_tiles = []
        exit_tiles2 = []
        for exit in self.h_tiledlayers.exit_data:
            exit_tiles.append(exit[0])
            exit_tiles2.append(exit[0]+1)
        eid = -1
        if current_tile in exit_tiles:
            eid = exit_tiles.index(current_tile)
        elif current_tile in exit_tiles2:
            eid = exit_tiles2.index(current_tile)
        return eid
    
    def UpdateMotion(self):
        
        if self.exiting or self.dying:
            return
        
        init_tile = self.h_tiledlayers.map_size[0]*((self.y+4)/self.h_tiledlayers.tilesize)+(self.x/self.h_tiledlayers.tilesize)
        ts = self.h_tiledlayers.tilesize
        
        # Compute inertia
        vx = self.speed*(self.control.r-self.control.l)
        if self.ground_state == False:
            self.vy = min(self.vy + 1, 5)
        else:
            self.vy = 0
            self.wvy = 0.0
        
        # Player Jump
        if self.control.jump:
            if self.ground_state == True:
                self.vy = -8
                self.ground_state = False
            self.control.jump = False
        
        # Equations of motion
        self.x = self.x + vx
        self.y = self.y + self.vy
        
        # Don't fall off the map
        self.x = min(self.x, (self.h_tiledlayers.map_size[0]-1)*16)
        self.x = max(self.x, 0+(self.sx/2))
        self.y = min(self.y, (self.h_tiledlayers.map_size[1]-1)*16)
        self.y = max(self.y, 0+16)
        
        # Handle all collisions off solid objects
        (self.x,self.y) = self.HandleObjectWallCollision((self.x,self.y),(vx,self.vy))
        #(x2,y2) = self.h_tiledlayers.HandleObjectWallCollision((self.x,self.y),(20,26),(vx,self.vy))
        
        # Handle pushing boxes
        new_tile = self.h_tiledlayers.map_size[0]*((self.y+4)/16)+(self.x/16)
        idlist = self.h_tiledlayers.FindMechanisms(new_tile,'block')
        ground_state_on_block = False
        for id in idlist:
            block = self.h_tiledlayers.movingblocks[id-100]
            checkx = self.x+(self.sx/2) > (block.x-ts/2) and self.x-(self.sx/2) < (block.x+ts/2)
            checky = self.y+(32/2) > (block.y-ts/2) and self.y-(32/2) < (block.y+ts/2)
            checkx2 = self.x+(self.sx/2) > (block.x-ts/2) and self.x-(self.sx/2) < (block.x+ts/2)
            checky2 = self.y+(32/2) >= (block.y-ts/2) and self.y+(32/2) < (block.y-(ts/2)+15)
            if checkx2 and checky2 and self.vy >= 0 and block.ground_state:
                ground_state_on_block = True
                self.y = block.y-(ts/2)-(32/2)
            elif checkx and checky:
                #if self.ground_state and self.control.use:
                if self.ground_state:
                    block.MoveBlock((vx,0))
            # re-enforce constraints back on player
            (self.x,self.y) = self.HandleBoxCollision((self.x,self.y),(vx,self.vy),block)
        
        # Handle platforms
        new_tile = self.h_tiledlayers.map_size[0]*((self.y+4)/16)+(self.x/16)
        idlist = self.h_tiledlayers.FindMechanisms(new_tile,'platform')
        ground_state_on_platform = False
        for id in idlist:
            platform = self.h_tiledlayers.platformcontrol.platforms[id-300]
            checkx2 = self.x+(self.sx/2) > (platform.x-ts/2) and self.x-(self.sx/2) < (platform.x+2*ts+ts/2)
            checky2 = self.y+(32/2) >= (platform.y-ts/2) and self.y+(32/2) < (platform.y-(ts/2)+15)
            if checkx2 and checky2 and self.vy >= 0:
                ground_state_on_platform = True
                self.y = platform.y-(ts/2)-(32/2)  
                    
        # Check if need to inform tilemap object layer of updates
        final_tile = self.h_tiledlayers.map_size[0]*((self.y+4)/16)+(self.x/16)
        if not init_tile == final_tile:
            self.h_tiledlayers.UpdateObj(init_tile,final_tile,0)
        
        # Handle up button control (leave level)
        if self.control.u > 0:
            eid = self.ExitCheck()
            if eid >= 0: # check for opening door/exit
                self.exiting = True
                self.x = self.h_tiledlayers.tilesize*(self.h_tiledlayers.exit_data[eid][0]%self.h_tiledlayers.map_size[0])+16
                self.y = self.h_tiledlayers.tilesize*(self.h_tiledlayers.exit_data[eid][0]/self.h_tiledlayers.map_size[0])
                self.gaittype = 'exit'
                self.gait = 0
                self.destination = [self.h_tiledlayers.exit_data[eid][1],self.h_tiledlayers.exit_data[eid][2]]
                return
        
        # Check for ground state
        leftfoottile = self.h_tiledlayers.map_size[0]*((self.y+16+1)/self.h_tiledlayers.tilesize)+((self.x-8)/self.h_tiledlayers.tilesize)
        rightfoottile = self.h_tiledlayers.map_size[0]*((self.y+16+1)/self.h_tiledlayers.tilesize)+((self.x+8)/self.h_tiledlayers.tilesize)
        if (self.vy >= 0 and (self.h_tiledlayers.occlayer[leftfoottile] == 1 or self.h_tiledlayers.occlayer[rightfoottile] == 1) or ground_state_on_block or ground_state_on_platform):
            self.ground_state = True
        else:
            self.ground_state = False
        
        # Check touching any hurt blocks
        idlist = self.h_tiledlayers.FindMechanisms(new_tile,'hurtblock')
        for id in idlist:
            block = self.h_tiledlayers.hurtblockcontrol.blocks[id-400]
            if block.type == 'lava':
                checkx = self.x+(self.sx/2) >= ts*(block.tile%self.h_tiledlayers.map_size[0])-1 and self.x-(self.sx/2) <= ts*(block.tile%self.h_tiledlayers.map_size[0])+ts+1
                checky = self.y+(32/2) >= ts*(block.tile/self.h_tiledlayers.map_size[0])+8 and self.y-(32/2) <= ts*(block.tile/self.h_tiledlayers.map_size[0])+ts+1
            else:
                checkx = self.x+(self.sx/2) >= ts*(block.tile%self.h_tiledlayers.map_size[0])-1 and self.x-(self.sx/2) <= ts*(block.tile%self.h_tiledlayers.map_size[0])+ts+1
                checky = self.y+(32/2) >= ts*(block.tile/self.h_tiledlayers.map_size[0])-1 and self.y-(32/2) <= ts*(block.tile/self.h_tiledlayers.map_size[0])+ts+1
            if checkx and checky:
                if block.switched_on:
                    if block.type == 'lava':
                        resources.soundfx['lava'].play()
                    else:
                        resources.soundfx['electric'].play()
                    self.kill()
        
        # Calculate other player data
        if vx > 0:
            self.heading = 'right'
        elif vx < 0:
            self.heading = 'left'
        
        if self.ground_state == False:
            self.gaittype = 'jump'
        elif vx == 0:
            self.gaittype = 'stand'
        else:
            self.gaittype = 'walk'
        
        if self.gaittype == 'walk' and self.gait_to == 0:
            self.gait_to = 3
            self.gait = self.gait + 1
            if self.gait > 3:
                self.gait = 0
        elif self.gaittype == 'stand' or self.gaittype == 'jump':
            self.gait = 0
            self.gait_to = 0
        self.gait_to = max(0,self.gait_to-1)
    
    def Draw(self,screen):
        tilecoords = resources.player_spritesheet_coords[self.heading][resources.player_spriteseq[self.gaittype][self.gait]]
        #screen.blit(resources.player_spritesheet, (self.x-(self.sizex/2)-self.h_camera.x+self.h_camera.w_view/2,self.y-(self.sizey/2)-self.h_camera.y+self.h_camera.h_view/2), area=tilecoords)
        screen.blit(resources.player_spritesheet, (self.x-(self.sizex/2)-self.h_camera.x+self.h_camera.w_view/2,self.y-12-self.h_camera.y+self.h_camera.h_view/2), area=tilecoords)

