#!/usr/bin/python

"""
resources.py - Load all resource data once 
"""

import os, sys
import pygame
from pygame.locals import *

class InitLevelData(object):
    def __init__(self):
        pass
        
    def load_level(self, filepath):
        
        f = open(filepath, "r")
        self.map_size = [int(i) for i in f.readline().split()]
        self.player_start_tile = [int(i) for i in f.readline().split()][0]
        numexits = [int(i) for i in f.readline().split()][0]
        self.exit_data = []
        for i in xrange(numexits):
            data = f.readline().split()
            self.exit_data.append((int(data[0]),data[1],int(data[2])))
        self.music = f.readline().split()[0]
        self.bgcolour = [int(i) for i in f.readline().split()]
        self.tilelayer_bg = []
        self.tilelayer_mg = []
        self.tilelayer_fg = []
        self.occlayer = []
        for j in xrange(self.map_size[1]):
            self.tilelayer_bg.extend([int(i) for i in f.readline().split()])
        for j in xrange(self.map_size[1]):
            self.tilelayer_mg.extend([int(i) for i in f.readline().split()])
        for j in xrange(self.map_size[1]):
            self.tilelayer_fg.extend([int(i) for i in f.readline().split()])
        for j in xrange(self.map_size[1]):
            self.occlayer.extend([int(i) for i in f.readline().split()])
        self.movingblockdata = [int(i) for i in f.readline().split()]
        numhurtblocks = [int(i) for i in f.readline().split()][0]
        self.hurtblockdata = []
        for i in xrange(numhurtblocks):
            data = f.readline().split()
            self.hurtblockdata.append((int(data[0]),data[1]))
        numplatforms = [int(i) for i in f.readline().split()][0]
        self.platformdata = []
        for i in xrange(numplatforms):
            data = f.readline().split()
            self.platformdata.append((int(data[0]),int(data[1]),int(data[2]),data[3],int(data[4])))
        numdoors = [int(i) for i in f.readline().split()][0]
        self.doordata = []
        for i in xrange(numdoors):
            data = f.readline().split()
            self.doordata.append((int(data[0]),data[1],data[2]))
        numbuttons = [int(i) for i in f.readline().split()][0]
        self.buttondata = []
        for i in xrange(numbuttons):
            data = f.readline().split()
            self.buttondata.append((int(data[0]),data[1],[]))
            nmech = int(data[2]) # number of mechanisms controlled
            for j in xrange(nmech):
                data = f.readline().split()
                self.buttondata[-1][2].append((data[0],int(data[1]))) # type, ind
        self.killerbatdata = [int(i) for i in f.readline().split()]
        f.close()

class GameFont(object):
    def __init__(self,mainpath,path,size):
        self.fontim = pygame.image.load(os.path.join(mainpath,'data','gfx',path)).convert()
        self.fontim.set_colorkey((255,0,255))
        
        keys = list('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ? = abcdefghijklmnopqrstuvwxyz+,-.!"#$%&`():')
        self.rects = {}
        for i in xrange(8):
            for j in xrange(10):
                k = keys[i*10+j]
                self.rects[k] = [j*size[0],i*size[1],size[0],size[1]]
        self.rects['i'][2] = size[1]/5
        self.rects['l'][2] = size[1]/5
    
    def RenderSentence(self, sentence, maxx, maxy, align='left'):
        sentsurf = pygame.Surface((maxx,maxy))
        sentsurf.fill((255,0,255))
        sentsurf.convert()
        sentsurf.set_colorkey((255,0,255))
        textheight = self.rects['A'][3]
        maxlines = maxy/textheight
        linedist = 0
        line = 0
        offsets = []
        if align in ['centre','vcentre']:
            for word in sentence.split(' '):
                letters = word.split()[0]+' '
                wordsize = sum([self.rects[let][2] for let in letters])
                if wordsize > maxx:
                    return False
                if (linedist+wordsize) >= maxx:
                    line += 1
                    offsets.append((maxx-linedist)/2)
                    linedist = 0
                    if line == maxlines:
                        return False
                for let in letters:
                    linedist += self.rects[let][2]
                if word[-1] == '\n':
                    line += 1
                    offsets.append((maxx-linedist)/2)
                    linedist = 0
            offsets.append((maxx-linedist)/2)
            voffset = maxy/2 - ((line+1)*textheight)/2
            linedist = 0
            line = 0
        for word in sentence.split(' '):
            letters = word.split()[0]+' '
            wordsize = sum([self.rects[let][2] for let in letters])
            if wordsize > maxx:
                return False
            if (linedist+wordsize) >= maxx:
                line += 1
                linedist = 0
                if line == maxlines:
                    return False
            for let in letters:
                if align == 'centre':
                    sentsurf.blit(self.fontim,(linedist+offsets[line],textheight*line),area=self.rects[let])
                elif align == 'vcentre':
                    sentsurf.blit(self.fontim,(linedist+offsets[line],textheight*line+voffset),area=self.rects[let])
                else:
                    sentsurf.blit(self.fontim,(linedist,textheight*line),area=self.rects[let])
                linedist += self.rects[let][2]
            if word[-1] == '\n':
                line += 1
                linedist = 0
        return sentsurf

def init(mainpath,screen_res):
    
    # Load background and tile layer images
    global tiles
    global tiles_coords
    
    tiles = pygame.image.load(os.path.join(mainpath,'data','gfx','tiles_2x.png')).convert()
    tiles.set_colorkey((255,0,255))
    tiles_coords = []
    for j in xrange(20):
        for i in xrange(16):
            tiles_coords.append((i*18+2, j*18+2, 16, 16))
    
    global tiles_coords_buttons
    tiles_coords_buttons = {}
    tiles_coords_buttons['mom'] = [(4*18+2, 14, 16, 20), (5*18+2, 18, 16, 16)]
    tiles_coords_buttons['perm'] = [(6*18+2, 14, 16, 20), (7*18+2, 18, 16, 16)]
    
    global tiles_doorseq
    tiles_doorseq = {}
    tiles_doorseq['vertical'] = [[27,31,30,29,28],[43,47,46,45,44],[62,62,62,61,60]]
    tiles_doorseq['horizontal'] = [[90,88,86,83,80],[91,89,87,84,81],[75,75,75,85,82]]
    
    global tiles_platform
    tiles_platform = {}
    tiles_platform['base_coords'] = [24,25,26]
    tiles_platform['chainbase'] = {}
    tiles_platform['chainbase']['left'] = (10*18+2,3*18+2,16,16)
    tiles_platform['chainbase']['right'] = (10*18,3*18+2,16,16)
    tiles_platform['chainlink'] = {}
    tiles_platform['chainlink']['left'] = (10*18+2,2*18+4+2,16,12)
    tiles_platform['chainlink']['right'] = (10*18,2*18+4+2,16,12)
    
    global tiles_hurtblockseq
    tiles_hurtblockseq = {}
    #tiles_hurtblockseq['lava'] = [78,79,79,95,94,94]
    tiles_hurtblockseq['lava'] = [78,79,79,78,94,94]
    tiles_hurtblockseq['elecadd'] = [0,2,4,6,8]
    
    # Load player sprites
    global player_spritesheet
    global player_spritesheet_coords
    global player_spriteseq
    
    player_spritesheet = pygame.image.load(os.path.join(mainpath,'data','gfx','player_2x.png')).convert()
    player_spritesheet.set_colorkey((255,0,255))
    player_spritesheet_coords = {}
    player_spritesheet_coords['left'] = []
    player_spritesheet_coords['right'] = []
    for i in xrange(2):
        player_spritesheet_coords['right'].append((i*20, 0, 20, 28))
        player_spritesheet_coords['left'].append((i*20+6*20, 0, 20, 28))
    for i in xrange(4):
        player_spritesheet_coords['right'].append((i*20, 28, 20, 28))
        player_spritesheet_coords['left'].append((i*20+4*20, 28, 20, 28))
    player_spriteseq = {}
    player_spriteseq['stand'] = [0]
    player_spriteseq['walk'] = [2,3,4,5]
    player_spriteseq['jump'] = [3]
    player_spriteseq['exit'] = [0]
    
    # Killer Bat sprites
    global bat_spritesheet
    global bat_spritesheet_coords
    global bat_spriteseq
    
    bat_spritesheet = pygame.image.load(os.path.join(mainpath,'data','gfx','bat_2x.png')).convert()
    bat_spritesheet.set_colorkey((255,0,255))
    bat_spritesheet_coords = {}
    bat_spritesheet_coords['left'] = []
    bat_spritesheet_coords['right'] = []
    bat_spritesheet_coords['left'] = []
    bat_spritesheet_coords['right'] = []
    for i in xrange(4):
        bat_spritesheet_coords['right'].append((i*20, 0, 20, 20))
        bat_spritesheet_coords['left'].append((i*20+80, 0, 20, 20))
    bat_spriteseq = {}
    bat_spriteseq['fly'] = [0,1,2,3]
    bat_spriteseq['death'] = [0]
    
    # Load level data
    global initleveldata
    
    initleveldata = {}
    
    initleveldata["level_computerroom"] = InitLevelData()
    initleveldata["level_computerroom"].load_level(os.path.join(mainpath,"data","lvl","level_computerroom.txt"))
    
    initleveldata["level_dragonroom"] = InitLevelData()
    initleveldata["level_dragonroom"].load_level(os.path.join(mainpath,"data","lvl","level_dragonroom.txt"))
    
    initleveldata["level_basicmechanisms"] = InitLevelData()
    initleveldata["level_basicmechanisms"].load_level(os.path.join(mainpath,"data","lvl","level_basicmechanisms.txt"))
    initleveldata["level_basiclava"] = InitLevelData()
    initleveldata["level_basiclava"].load_level(os.path.join(mainpath,"data","lvl","level_basiclava.txt"))
    initleveldata["level_basicelectric"] = InitLevelData()
    initleveldata["level_basicelectric"].load_level(os.path.join(mainpath,"data","lvl","level_basicelectric.txt"))
    initleveldata["level_basicbats"] = InitLevelData()
    initleveldata["level_basicbats"].load_level(os.path.join(mainpath,"data","lvl","level_basicbats.txt"))
    initleveldata["level_theroundabout"] = InitLevelData()
    initleveldata["level_theroundabout"].load_level(os.path.join(mainpath,"data","lvl","level_theroundabout.txt"))
    
    initleveldata["level_mediumbats"] = InitLevelData()
    initleveldata["level_mediumbats"].load_level(os.path.join(mainpath,"data","lvl","level_mediumbats.txt"))
    
    initleveldata["level_thelift"] = InitLevelData()
    initleveldata["level_thelift"].load_level(os.path.join(mainpath,"data","lvl","level_thelift.txt"))
    
    initleveldata["level_getscroll_blue"] = InitLevelData()
    initleveldata["level_getscroll_blue"].load_level(os.path.join(mainpath,"data","lvl","level_getscroll_blue.txt"))
    initleveldata["level_getscroll_green"] = InitLevelData()
    initleveldata["level_getscroll_green"].load_level(os.path.join(mainpath,"data","lvl","level_getscroll_green.txt"))
    initleveldata["level_getscroll_red"] = InitLevelData()
    initleveldata["level_getscroll_red"].load_level(os.path.join(mainpath,"data","lvl","level_getscroll_red.txt"))
    
    # Sound Data
    global soundfx
    soundfx = {}
    soundfx['lava'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','acid_burn_0.ogg'))
    soundfx['bat'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','qubodupSqueakyRatAttack.ogg'))
    soundfx['electric'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','248251__jameswrowles__electrocution.ogg'))
    soundfx['button'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','31589__freqman__buttons01.ogg'))
    
    # Music Data
    global musicpaths
    musicpaths = {}
    musicpaths['title'] = os.path.join(mainpath,'data','music','pippin_the_hunchback.ogg')
    musicpaths['musictest'] = os.path.join(mainpath,'data','music','pippin_the_hunchback.ogg')
    musicpaths['gotscroll'] = os.path.join(mainpath,'data','music','pippin_the_hunchback.ogg')
    
    # pre-sets and controls
    global controlmap
    controlmap = {}
    controlmap['R'] = K_RIGHT
    controlmap['L'] = K_LEFT
    controlmap['D'] = K_DOWN
    controlmap['U'] = K_UP
    controlmap['Jump'] = K_z
    controlmap['Use'] = K_x
    
    # Fonts and text
    global gamefont
    gamefont = GameFont(mainpath,'broddmin_5x10_shad_2x.png',(10,20))
    gamefontred = GameFont(mainpath,'broddmin_5x10_shad_red_2x.png',(10,20))
    
    ####################
    # Title Screen Stuff
    global titlescreen_surfs
    titlescreen_surfs = {}
    
    titlescreen_surfs['bg'] = pygame.image.load(os.path.join(mainpath,'data','gfx','title_bg_2x.png')).convert()
    titlescreen_surfs['bg'].set_colorkey((255,0,255))
    
    titlescreen_surfs['newgame_white'] = gamefont.RenderSentence("New Game", screen_res[0], screen_res[1]/2, align='vcentre')
    titlescreen_surfs['continue_white'] = gamefont.RenderSentence("Continue", screen_res[0], screen_res[1]/2, align='vcentre')
    titlescreen_surfs['newgame_red'] = gamefontred.RenderSentence("New Game", screen_res[0], screen_res[1]/2, align='vcentre')
    titlescreen_surfs['continue_red'] = gamefontred.RenderSentence("Continue", screen_res[0], screen_res[1]/2, align='vcentre')
    
    ####################
    # Pause Screen Stuff
    global pausescreen_surfs
    pausescreen_surfs = {}
    
    pausescreen_full = pygame.image.load(os.path.join(mainpath,'data','gfx','paused_2x.png')).convert()
    #pausescreen_full.set_colorkey((255,0,255))
    
    pausescreen_surfs['help'] = pygame.image.load(os.path.join(mainpath,'data','gfx','help_2x.png')).convert()
    pausescreen_surfs['help'].set_colorkey((255,0,255))
    
    pausescreen_surfs['bg'] = pygame.Surface((90,80))
    pausescreen_surfs['bg'].fill((0,0,0))
    pausescreen_surfs['bg'].set_colorkey((255,0,255))
    pausescreen_surfs['bg'].blit(pausescreen_full,(0,0),area=(0,0,90,80))
    
    pausescreen_surfs['resume_on'] = pygame.Surface((90,10))
    pausescreen_surfs['resume_on'].fill((0,0,0))
    pausescreen_surfs['resume_on'].set_colorkey((255,0,255))
    pausescreen_surfs['resume_on'].blit(pausescreen_full,(0,0),area=(0,90,90,10))
    pausescreen_surfs['resume_off'] = pygame.Surface((90,10))
    pausescreen_surfs['resume_off'].fill((0,0,0))
    pausescreen_surfs['resume_off'].set_colorkey((255,0,255))
    pausescreen_surfs['resume_off'].blit(pausescreen_full,(0,0),area=(0,80,90,10))
    
    pausescreen_surfs['reset_on'] = pygame.Surface((90,10))
    pausescreen_surfs['reset_on'].fill((0,0,0))
    pausescreen_surfs['reset_on'].set_colorkey((255,0,255))
    pausescreen_surfs['reset_on'].blit(pausescreen_full,(0,0),area=(0,110,90,10))
    pausescreen_surfs['reset_off'] = pygame.Surface((90,10))
    pausescreen_surfs['reset_off'].fill((0,0,0))
    pausescreen_surfs['reset_off'].set_colorkey((255,0,255))
    pausescreen_surfs['reset_off'].blit(pausescreen_full,(0,0),area=(0,100,90,10))
    
    pausescreen_surfs['quit_on'] = pygame.Surface((90,10))
    pausescreen_surfs['quit_on'].fill((0,0,0))
    pausescreen_surfs['quit_on'].set_colorkey((255,0,255))
    pausescreen_surfs['quit_on'].blit(pausescreen_full,(0,0),area=(0,130,90,10))
    pausescreen_surfs['quit_off'] = pygame.Surface((90,10))
    pausescreen_surfs['quit_off'].fill((0,0,0))
    pausescreen_surfs['quit_off'].set_colorkey((255,0,255))
    pausescreen_surfs['quit_off'].blit(pausescreen_full,(0,0),area=(0,120,90,10))
    
    ####################
    # Cutscene Stuff
    
    # Actual cutscene content
    global logocutscene_data
    logocutscene_data = [[ ['team_chimera',[]] ],'none','titlescene',[]]
    
    global storycutscene_data
    storycutscene_data = [[ ['otrux_dragon',['otrux_dragon_text001','otrux_dragon_text002','otrux_dragon_text003']], \
        ['castle',['castle_text001','castle_text002']], \
        ['otrux_dragon2',['otrux_dragon2_text001','otrux_dragon2_text002','otrux_dragon2_text003']], \
        ['otrux_computer',['otrux_computer_text001','otrux_computer_text002','otrux_computer_text003']], \
        ['otrux_computer2',['otrux_computer2_text001','otrux_computer2_text002']], \
        ['otrux_grave',['otrux_grave_text001']], \
        ['otrux_mech1',['otrux_mech1_text001']], \
        ['player',['player_text001']], \
        ['scrolls',['scrolls_text001']], \
        ], \
        'title','maingame',[True,"level_computerroom",-1]]
    
    global wingamescene_data
    wingamescene_data = [[ ['win001',['win001_text001','win001_text002','win001_text003','win001_text004']], \
        ['win002',['win002_text001','win002_text002']], \
        ['win003',['win003_text001']], \
        ], \
        'none','titlescene',[]]
    
    # Got Data Scroll
    global gotscroll_surfs
    scrolls_img = pygame.image.load(os.path.join(mainpath,'data','gfx','datascrolls_8x.png')).convert()
    scrolls_img.set_colorkey((255,0,255))
    
    text_red = gamefont.RenderSentence("You got the red data scroll!", screen_res[0], screen_res[1]/2, align='vcentre')
    text_green = gamefont.RenderSentence("You got the green data scroll!", screen_res[0], screen_res[1]/2, align='vcentre')
    text_blue = gamefont.RenderSentence("You got the blue data scroll!", screen_res[0], screen_res[1]/2, align='vcentre')
    
    gotscroll_surfs = {}
    gotscroll_surfs['scroll_red'] = pygame.Surface((screen_res[0],screen_res[1]))
    gotscroll_surfs['scroll_red'].fill((0,0,0))
    gotscroll_surfs['scroll_red'].blit(scrolls_img,(screen_res[0]/2-40,screen_res[1]/2-60),area=(80,0,80,80))
    gotscroll_surfs['scroll_red'].blit(text_red,(0,screen_res[1]/2))
    gotscroll_surfs['scroll_green'] = pygame.Surface((screen_res[0],screen_res[1]))
    gotscroll_surfs['scroll_green'].fill((0,0,0))
    gotscroll_surfs['scroll_green'].blit(scrolls_img,(screen_res[0]/2-40,screen_res[1]/2-60),area=(160,0,80,80))
    gotscroll_surfs['scroll_green'].blit(text_green,(0,screen_res[1]/2))
    gotscroll_surfs['scroll_blue'] = pygame.Surface((screen_res[0],screen_res[1]))
    gotscroll_surfs['scroll_blue'].fill((0,0,0))
    gotscroll_surfs['scroll_blue'].blit(scrolls_img,(screen_res[0]/2-40,screen_res[1]/2-60),area=(0,0,80,80))
    gotscroll_surfs['scroll_blue'].blit(text_blue,(0,screen_res[1]/2))
    
    # Initial cutscenes
    global cutscenesurfs
    cutscenesurfs = {}
    
    cutscenesurfs['team_chimera'] = pygame.image.load(os.path.join(mainpath,'data','gfx','team_chimera_5x.png')).convert()
    cutscenesurfs['team_chimera'].set_colorkey((255,0,255))
    
    cutscenesurfs['otrux_dragon'] = pygame.image.load(os.path.join(mainpath,'data','gfx','cutscene_otruxdragon_2x.png')).convert()
    cutscenesurfs['otrux_dragon'].set_colorkey((255,0,255))
    cutscenesurfs['otrux_dragon_text001'] = gamefont.RenderSentence("The great dragon slept while Otrux the wizard collected his data on the beast ..", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['otrux_dragon_text002'] = gamefont.RenderSentence("how often it slept, the length of its claws and the temperature of its breath.", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['otrux_dragon_text003'] = gamefont.RenderSentence("So it had been night after night for the last three years, ever since the beast had descended upon the kingdom of Arvendon and made the King`s treasure room it`s resting place.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['castle'] = pygame.image.load(os.path.join(mainpath,'data','gfx','castle_2x.png')).convert()
    cutscenesurfs['castle'].set_colorkey((255,0,255))
    cutscenesurfs['castle_text001'] = gamefont.RenderSentence("The kingdom had been the most prosperous in the lands in it`s former glory days, but without it`s treasure, the kingdom had declined.", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['castle_text002'] = gamefont.RenderSentence("The king placed a heft bounty on the beast`s head .. many a knight would come to challenge the dragon, but all in vain.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['otrux_dragon2'] = pygame.image.load(os.path.join(mainpath,'data','gfx','cutscene_otruxdragon2_2x.png')).convert()
    cutscenesurfs['otrux_dragon2'].set_colorkey((255,0,255))
    cutscenesurfs['otrux_dragon2_text001'] = gamefont.RenderSentence("Otrux would observe such encounters carefully ..", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['otrux_dragon2_text002'] = gamefont.RenderSentence("as limb was torn from limb and flesh melted and scorched, Otrux would observe something useful, something he hadn`t seen the dragon do before, refine his knowledge on previously measured phenomena.", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['otrux_dragon2_text003'] = gamefont.RenderSentence("Everything was recorded, every minutiae .. something would prove useful.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['otrux_computer'] = pygame.image.load(os.path.join(mainpath,'data','gfx','cutscene_otruxcomputer_2x.png')).convert()
    cutscenesurfs['otrux_computer'].set_colorkey((255,0,255))
    cutscenesurfs['otrux_computer_text001'] = gamefont.RenderSentence("Otrux constructed as vast mechanical device, a system of great gears and pulleys, clocks and counters, a `computational engine`, he called it.", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['otrux_computer_text002'] = gamefont.RenderSentence("Night after night Otrux fed his data into the machine. The gears turned and clocks ticked ..", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['otrux_computer_text003'] = gamefont.RenderSentence("The machine searched tirelessly for something, a pattern in the measurements that would hold the key to defeating the dragon and ridding the kingdom of its menace.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['otrux_computer2'] = pygame.image.load(os.path.join(mainpath,'data','gfx','cutscene_otruxcomputer2_2x.png')).convert()
    cutscenesurfs['otrux_computer2'].set_colorkey((255,0,255))
    cutscenesurfs['otrux_computer2_text001'] = gamefont.RenderSentence("After three years, the clocks of the computational engine stopped and a chime rang out from within. This was the moment Otrux had been waiting for.", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['otrux_computer2_text002'] = gamefont.RenderSentence("The measurements had been analysed and an answer had been found. Finally the kingdom could rest again.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['otrux_grave'] = pygame.image.load(os.path.join(mainpath,'data','gfx','grave_2x.png')).convert()
    cutscenesurfs['otrux_grave'].set_colorkey((255,0,255))
    cutscenesurfs['otrux_grave_text001'] = gamefont.RenderSentence("But on the eve Otrux`s announcement, time finally caught up with him, and he died of old age.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['otrux_mech1'] = pygame.image.load(os.path.join(mainpath,'data','gfx','cutscene_mech1.png')).convert()
    cutscenesurfs['otrux_mech1'].set_colorkey((255,0,255))
    cutscenesurfs['otrux_mech1_text001'] = gamefont.RenderSentence("The wizard`s data was lost inside the vast mechanical dungeon below the central control panel, the working components of his machine.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['player'] = pygame.image.load(os.path.join(mainpath,'data','gfx','cutscene_player.png')).convert()
    cutscenesurfs['player'].set_colorkey((255,0,255))
    cutscenesurfs['player_text001'] = gamefont.RenderSentence("As Otrux`s humble apprentice, the job of finding Otrux`s missing data and the answer to the problem of the dragon falls to you.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['scrolls'] = pygame.image.load(os.path.join(mainpath,'data','gfx','datascrolls_8x.png')).convert()
    cutscenesurfs['scrolls'].set_colorkey((255,0,255))
    cutscenesurfs['scrolls_text001'] = gamefont.RenderSentence("All of Otrux`s data was recorded on three scrolls which were lost in the depths of his machine. Once these are recovered and input to the computer, the dragon riddle will be solved.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['dungeon1'] = pygame.image.load(os.path.join(mainpath,'data','gfx','datascrolls_8x.png')).convert()
    cutscenesurfs['dungeon1'].set_colorkey((255,0,255))
    cutscenesurfs['dungeon1_text001'] = gamefont.RenderSentence("All of Otrux`s data was recorded on three scrolls which were lost in the depths of his machine. Once these are recovered and input to the computer, the dragon riddle will be solved.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['win001'] = pygame.image.load(os.path.join(mainpath,'data','gfx','cutscene_otruxcomputer2_2x.png')).convert()
    cutscenesurfs['win001'].set_colorkey((255,0,255))
    cutscenesurfs['win001_text001'] = gamefont.RenderSentence("Finally, Otrux`s data has been recovered and you load the scrolls into the control panel.", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['win001_text002'] = gamefont.RenderSentence("The machine hums and clicks and after several minutes produces a scroll containing the answer.", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['win001_text003'] = gamefont.RenderSentence("The scroll contains a single sentence ..", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['win001_text004'] = gamefontred.RenderSentence("`Maybe you should just be friends with the dragon`", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['win002'] = pygame.image.load(os.path.join(mainpath,'data','gfx','cutscene_player.png')).convert()
    cutscenesurfs['win002'].set_colorkey((255,0,255))
    cutscenesurfs['win002_text001'] = gamefont.RenderSentence("You sit in stunned silence.", screen_res[0], screen_res[1], align='vcentre')
    cutscenesurfs['win002_text002'] = gamefont.RenderSentence("After a while you think about it. Maybe you should .. dragons are pretty cool.", screen_res[0], screen_res[1], align='vcentre')
    
    cutscenesurfs['win003'] = pygame.image.load(os.path.join(mainpath,'data','gfx','cutscene_win.png')).convert()
    cutscenesurfs['win003'].set_colorkey((255,0,255))
    cutscenesurfs['win003_text001'] = gamefont.RenderSentence("Congratulations!", screen_res[0], screen_res[1], align='vcentre')
    